#lang slideshow
(require slideshow/code
         slideshow/play
         "utils/utils.ss"
         "utils/colors.ss"
         "utils/alg.ss")

(define (align . l) (apply para #:width (* 0.95 client-w) l))

(define (records-part n)
  (part #:chapter "Records" n))
(define (encoding-part n)
  (part #:chapter "Encoding" n))

;; ------------------------------------------------------------
(records-part 1)

(slide
 #:title "Records"
 (tt* "struct Posn {"
      "  int x, y;"
      "}"
      " "
      "Posn p = new Posn();"
      "p.x = 0;"
      "p.y = 0;"))

(slide
 #:title "Records"
 (grammar-table
  (list (nonterm "Expr") eqls (nonterm "Num") (blank)
        (blank)          -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {* #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code #,(nonterm "Sym")) (blank)
        (blank)          -or- (code {lambda {#,(nonterm "Sym")} #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {#,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {record {#,(nonterm "Sym") #,(nonterm "Expr")} ...}) new-label
        (blank)          -or- (code {get #,(nonterm "Expr") #,(nonterm "Sym")}) new-label
        (blank)          -or- (code {set #,(nonterm "Expr") #,(nonterm "Sym") #,(nonterm "Expr")}) new-label)))

(slide
 #:title "Record Programs"
 'alts
 (list
  (list
   (para
    (code {let {[r {record {x 5}
                           {y 2}}]}
            {get r x}}
          code:blank
          #,sym:implies 5)))
  (list
   (para
    (code {let {[r {record {x 5}
                           {y 2}}]}
            {get r y}}
          code:blank
          #,sym:implies 2)))
  (list
   (para
    (code {let {[r {record {x 5}
                           {y {+ 1 1}}}]}
            {get r y}}
          code:blank
          #,sym:implies 2)))
  (list
   (para
    (code {let {[mk {lambda {v}
                      {record {x {+ v 1}}
                              {y {+ v 2}}}}]}
            {get {mk 2} x}}
          code:blank
          #,sym:implies 3)))
  (list
   (para
    (code {get {record {x 1}
                       {y 2}}
               x}
          code:blank
          #,sym:implies 1)))
  (list
   (para
    (code {record {x 1}
                  {y 2}}
          code:blank
          #,sym:implies #,(t "... a record ..."))))
  (list
   (para
    (code {set {record {x 1}
                       {y 2}}
               x
               5}
          code:blank
          #,sym:implies #,(para #:fill? #f "... a record with" (code x) "as" (code 5) "..."))))))

(slide
 #:title "Record Expressions & Values"
 (scode
  (define-type ExprC
    ....
    [recordC (ns : (listof symbol))
             (args : (listof ExprC))]
    [getC (rec : ExprC)
          (n : symbol)]
    [setC (rec : ExprC)
          (n : symbol)
          (val : ExprC)])
  code:blank
  (define-type Value
    [numV (n : number)]
    [closV (arg : symbol)
           (body : ExprC)
           (env : Env)]
    [recV (ns : (listof symbol))
          (vs : (listof Value))])))

;; ----------------------------------------
(records-part 2)

(slide
 #:title "Parsing Records"
 (scode
  (define (parse [s : s-expression]) : ExprC
    (cond
     ....
     [(s-exp-match? '{record {SYMBOL ANY} ...} s)
      (recordC (map (lambda (l) 
                      (s-exp->symbol
                       (first (s-exp->list l))))
                    (rest (s-exp->list s)))
               (map (lambda (l) 
                      (parse
                       (second (s-exp->list l))))
                    (rest (s-exp->list s))))]
     ....))))

(define (rinterp p #:suffix [suffix #f] #:scale [s 0.85])
  (list
   (align
    (scale
     (let ([c (code (define (interp [a : ExprC] [env : Env]) : Value
                      (type-case ExprC a
                        ...
                        #,p
                        ...)))])
       (if suffix
           (code #,c
                 #,suffix)
           c))
     s))))


(define rimpl-title (hbl-append (code interp) (titlet " for Records")))

(slide
 #:title rimpl-title
 #:layout 'top
 'alts~
 (list
  (rinterp
   (code
    [recordC (ns vs)
             ...]))
  (rinterp
   (code
    [recordC (ns vs)
             ... ns ...
             ... (map (lambda (v) (interp v env))
                      vs) ...]))
  (rinterp
   (code
    [recordC (ns vs)
             (recV ns
                   (map (lambda (v) (interp v env))
                        vs))]))
  
  (rinterp
   (code
    [getC (r n)
          ...]))
  (rinterp
   (code
    [getC (r n)
          ... (interp r env)
          ... n ...]))
  (rinterp
   (code
    [getC (r n)
          (type-case Value (interp r env)
            [recV (ns vs)
                  ... n ...]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [getC (r n)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (find n ns vs)]
            [else (error 'interp "not a record")])])
   #:suffix
   (code code:blank
         code:blank
         find : (symbol (listof symbol) (listof Value) -> Value)))


  (rinterp
   (code
    [setC (r n v)
          ...]))
  (rinterp
   (code
    [setC (r n v)
          ... (interp r env)
          ... n
          ... (interp v env) ...]))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  ... n
                  ... (interp v env) ...]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  ... (update n
                              (interp v env)
                              ns
                              vs) ...]
            [else (error 'interp "not a record")])])
   #:suffix
   (code code:blank
         code:blank
         update : (symbol Value (listof symbol) (listof Value) 
                          -> (listof Value))))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (recV ns
                        (update n
                                (interp v env)
                                ns
                                vs))]
            [else (error 'interp "not a record")])]))))

;; ----------------------------------------
(records-part 3)

(slide
 #:title "Functional Record Update"
 (para
  (code {let {[r1 {record {a {+ 1 1}}
                          {b {+ 2 2}}}]}
          {let {[r2 {set r1 a 5}]}
            {+ {get r1 a} 
               {get r2 a}}}}))
 'next
 (para sym:implies (code 7))
 'next
 (para (code set) "creates a new record with the new field value")
 (ghost (para "x"))
 (blank)
 (para "This is called" (bit "functional update")))

(slide
 #:title "Imperative Record Update"
 (para
  (code {let {[r1 {record {a {+ 1 1}}
                          {b {+ 2 2}}}]}
          {let {[r2 {set r1 a 5}]}
            {+ {get r1 a} 
               {get r2 a}}}}))
 'next
 (para sym:implies (code 10))
 'next
 (para "assuming that" (code set) "mutates a record's field")
 (para #:align 'right "... like a box")
 (blank)
 (para "This is called" (bit "imperative update")))


(slide
 #:title "Record Expressions & Values"
 (scode
  (define-type Value
    ....
    [recV (ns : (listof symbol))
          (vs : (listof (boxof Value)))])))

(define mrimpl-title (hbl-append (code interp) (titlet " for Mutable Records")))

(slide
 #:title mrimpl-title
 #:layout 'top
 'alts~
 (list
  (rinterp
   (code
    [recordC (ns vs)
             (recV ns
                   (map (lambda (v) (interp v env))
                        vs))]))
  (rinterp
   (code
    [recordC (ns vs)
             (recV ns
                   (map (lambda (v) (box (interp v env)))
                        vs))]))
  
  (rinterp
   (code
    [getC (r n)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (find n ns vs)]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [getC (r n)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (unbox (find n ns vs))]
            [else (error 'interp "not a record")])])
   #:suffix
   (code code:blank
         code:blank
         find : (symbol (listof symbol) (listof (boxof Value))
                        -> (boxof Value))))

  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (recV ns
                        (update n
                                (interp v env)
                                ns
                                vs))]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (let ([bx (find n ns vs)])
                    (recV ns
                          ... (interp v env)
                          ...
                          ...))]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (let ([bx (find n ns vs)])
                    (begin
                      (set-box! bx (interp v env))
                      (recV ns
                            ...)))]
            [else (error 'interp "not a record")])]))
  (rinterp
   (code
    [setC (r n v)
          (type-case Value (interp r env)
            [recV (ns vs)
                  (let ([bx (find n ns vs)])
                    (begin
                      (set-box! bx (interp v env))
                      (recV ns
                            vs)))]
            [else (error 'interp "not a record")])]))))

;; ------------------------------------------------------------
(encoding-part 1)

(let ([rhs (code 5)]
      [body (code {+ x 6})]
      [name (code x)])
  (slide
   #:title "Binding Constructs"
   (code
    {let {[#,name #,rhs]}
      #,body}
    code:blank
    code:blank
    {let {[f {lambda {#,name}
               #,body}]}
      {f #,rhs}})))

(define def (let ([eq (t "=")]
                  [def (scale/improve-new-text (t "def") 0.35)])
              (refocus (vc-append (- (/ (pict-height eq) 3))
                                  def
                                  eq)
                       eq)))

(define (same-progs #:rhs [rhs (code 5)]
                    #:body [body (code {+ x 6})]
                    #:name [name (code x)])
  (list
   (code
    {let {[#,name #,rhs]}
      #,body}
    code:blank
    code:blank
    {{lambda {#,name}
       #,body}
     #,rhs})))


(slide
 #:title (hbl-append (titlet "Encoding ") (code let))
 (para "These programs are the same:")
 (blank)
 'alts
 (list
  (same-progs)
  (same-progs #:body (code _body))
  (same-progs #:body (code _body) #:rhs (code _rhs))
  (same-progs #:body (code _body) #:rhs (code _rhs) #:name (code _name)))
 'next
 (blank)
 (blank)
 (code (test (parse '{let {[x 5]} {+ x 6}})
             (appC (lamC 'x (plusC (idC 'x) (numC 6)))
                   (numC 5)))))

(define (mult-args #:body [body (code {+ x y})]
                   #:x [x (code x)]
                   #:y [y (code y)])
  (list
   (code
    {let {[f {lambda {#,x #,y}
               #,body}]}
      {f 1 2}}
    code:blank
    code:blank
    {let {[f {lambda {#,x}
               {lambda {#,y}
                 #,body}}]}
      {{f 1} 2}})))

(slide
 #:title "Encoding Multiple Arguments"
 'alts
 (list
  (mult-args)
  (mult-args #:body (code _body)))
 'next
 (blank)
 (para "This transformation is called" (bit "currying")))

(encoding-part 2)

(slide
 #:title (hbl-append (titlet "Encoding ") (code if))
 'alts
 (list
  (list
   (code {if _tst
             _thn
             _els}))
  (list
   (code {if* _tst
              {lambda {d} _thn}
              {lambda {d} _els}}))
  (list
   (code {{if* _tst
               {lambda {d} _thn}
               {lambda {d} _els}}
          0})
   'next
   (blank)
   (blank)
   (code true #,def {lambda {x} {lambda {y} x}}
         false #,def {lambda {x} {lambda {y} y}})
   'next
   (blank)
   (code {{{{_tst 
             {lambda {d} _thn}}
            {lambda {d} _els}}}
          0}))))

(define sym::equals
  (lbl-superimpose (ghost sym:implies) (t "=")))

(slide
 #:title "Encoding Pairs"
 'alts
 (list
  (list (code {cons 1 empty}))
  (list (code {pair 1 0}))
  (list (code {pair _f _r}))
  (list (code {lambda .... _f _r}))
  (list (code {lambda {sel} {{sel _f} _r}})
        'next
        (blank)
        (code pair #,def {lambda {x}
                           {lambda {y}
                             {lambda {sel} {{sel x} y}}}}
              fst #,def {lambda {p} {p true}}
              snd #,def {lambda {p} {p false}})
        'next
        (blank)
        (code {fst {{pair 1} 0}}
              #,sym:implies {fst {lambda {sel} {{sel 1} 0}}}
              #,sym:implies {{lambda {sel} {{sel 1} 0}} true}
              #,sym:implies {{true 1} 0}
              #,sym::equals {{{lambda {x} {lambda {y} x}} 1} 0}
              #,sym:implies {{lambda {y} 1} 0}
              #,sym:implies 1))))


;; ------------------------------------------------------------
(encoding-part 3)

(slide
 #:title "λ-Calculus Grammar"
 'alts
 (list
  (list
   (grammar-table
    (list (nonterm "Expr") eqls (nonterm "Sym") (blank)
          (blank)        -or- (code {#,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)        -or- (code {lambda {#,(nonterm "Sym")} #,(nonterm "Expr")}) (blank))))
  (list
   (grammar-table
    (list (nonterm "Expr") eqls (nonterm "Sym") (blank)
          (blank)        -or- (code {#,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
          (blank)        -or- (code (λ (#,(nonterm "Sym")) #,(nonterm "Expr"))) (blank)))))
 'next
 (blank)
 (blank)
 (code true #,def (λ (x) (λ (y) x))
       false #,def (λ (x) (λ (y) y))))

;; ------------------------------------------------------------
(encoding-part 4)

(define (nalign p) (para #:width (* client-w 0.9) p))

(define (_1 p)
  (hbl-append p (text "1" `(subscript . ,(current-main-font)) (current-font-size))))
  
(define f_1 (_1 (code f)))
(define f_N (hbl-append (code f) (text "N" `(italic subscript . ,(current-main-font)) (current-font-size))))

(slide
 #:title "Encoding Numbers"
 'alts
 (list
  (list (nalign (code zero #,def (λ (x) (λ (y) y)))))
  (list (nalign (code zero #,def (λ (f) (λ (y) y))))))
 'next
 (nalign (code one #,def (λ (f) (λ (y) {f y}))))
 'next
 (nalign (code two #,def (λ (f) (λ (y) {f {f y}}))))
 'next
 (nalign (code three #,def (λ (f) (λ (y) {f {f {f y}}}))))
 'next
 (blank)
 (nalign (code _N #,def (λ (f) (λ (y) {#,f_1 ... {#,f_N y}})))))

(define zero
  (frame
   (hbl-append (scale (code (λ (f) (λ (x) x))) 0.85)
               (let ([c (ghost (code x))])
                 (inset c 0 0 (- (pict-width c)) 0)))))

(slide
 #:title "Incrementing a Number"
 'alts
 (list
  (list (code add1 #,def (λ (n)
                           ...)))
  (list (code add1 #,def (λ (n)
                           (λ (f)
                             (λ (x) ...)))))
  (list (code add1 #,def (λ (n)
                           (λ (f)
                             (λ (x) ... {{n f} x} ...)))))
  (list (code add1 #,def (λ (n)
                           (λ (f)
                             (λ (x) {f {{n f} x}}))))))
 'next
 (blank)
 (code (add1 zero)
       #,sym:implies (λ (f)
                       (λ (x) {f {{zero f} x}}))
       #,sym::equals (λ (f)
                       (λ (x) {f {{#,zero f} x}}))
       #,sym:implies (λ (f)
                       (λ (x) {f x}))
       #,sym::equals one))

(define (_m p)
  (hbl-append p (text "m" `(subscript . ,(current-code-font)) (current-font-size))))

(define add1_1 (_1 (code add1)))
(define add1_m (_m (code add1)))

(slide
 #:title "Adding Numbers"
 (nalign (code add2 #,def (λ (n) {add1 {add1 n}})))
 'next
 (nalign (code add3 #,def (λ (n) {add1 {add1 {add1 n}}})))
 'next
 (blank)
 'alts
 (list
  (list
   (nalign (code add #,def (λ (n) (λ (m) {#,add1_1 ... {#,add1_m n}})))))
  (list
   (nalign (code add #,def (λ (n) (λ (m) {{m add1} n}))))
   (para "... because a number" (it "m") "applies some function"
       (it "m") "times to an argument")))
 'next
 (blank)
 (code {{add one} two}
       #,sym:implies {{two add1} one}
       #,sym:implies {add1 {add1 one}}
       #,sym:implies three))

(define addn_1 (_1 (code {add n})))
(define addn_m (_m (code {add n})))
       
(slide
 #:title "Multiplying Numbers"
 'alts
 (list
  (list
   (nalign (code mult #,def (λ (n) (λ (m) {#,addn_1 
                                           ... 
                                           {#,addn_m zero}})))))
  (list
   (nalign (code mult #,def (λ (n) (λ (m) {{m {add n}} zero}))))
   (blank)
   (para "... because" (code {add #,(it "n")}) "is a function that adds" (it "n")
         "to any number")
   (para "... and a number" (it "m") "applies some function"
       (it "m") "times to an argument"))))


(slide
 #:title "Testing for Zero"
 'alts
 (list
  (list
   (code iszero #,def (λ (n) ... true ... false ...)))
  (list
   (code iszero #,def (λ (n) {{n (λ (x) false)} 
                              true}))
   (blank)
   (para "because applying" (code (λ (x) false)) "zero"
         "times to" (code true) "produces" (code true) ", and"
         "applying it any other number of times produces" (code false))))
 'next
 (blank)
 'alts
 (list
  (list
   (code {iszero zero}
         #,sym:implies {{zero (λ (x) false)} true}
         #,sym:implies true))
  (list
   (code {iszero one}
         #,sym:implies {{one (λ (x) false)} true}
         #,sym:implies {(λ (x) false) true}
         #,sym:implies false))))

(slide
 #:title "Decrementing a Number"
 'alts
 (list
  (list
   (code sub1 #,def (λ (n)
                      (λ (f)
                        (λ (x) ...)))))
  (list
   (code sub1 #,def (λ (n)
                      (λ (f)
                        (λ (x) ... {{n f} x} ...))))
   'next
   (blank)
   (para "Too late! No way to undo a call to" (code f)))
  (list
   (code ... {{pair zero} one}
         ... {{pair one} two}
         ... {{pair two} three}
         ...
         ... {{pair #,(it "n-1")} #,(it "n")}))
  (list
   (table 3
          (list (code shift) def (code (λ (p)
                                         {{pair {snd p}} {add1 {snd p}}}))
                (code sub1) def (code (λ (n)
                                        {fst
                                         {{n shift} {{pair zero} zero}}})))
          ltl-superimpose ltl-superimpose
          gap-size gap-size)
   'next
   (blank)
   (para "And then subtraction is obvious..."))))

(define check (colorize (t " ✔") "forestgreen"))

(slide
 #:title "Encodings"
 (para "Using the minimal λ-calculus language we get")
 (item #:bullet check "functions")
 (item #:bullet check "local binding")
 (item #:bullet check "booleans")
 (item #:bullet check "numbers"))

;; ------------------------------------------------------------
(encoding-part 5)

(define fac-title "Factorial")
(define fac+rec-title "Make-Recursive and Factorial")

(define fac-layout 'top)

(slide
 #:title fac-title
 #:layout fac-layout
 (code (local [(define fac
                 (lambda (n)
                   (if (zero? n)
                       1
                       (* n (fac (- n 1))))))]
         (fac 10)))
 'next
 (blank)
 (blank)
 (para (code local) "binds both in the body"
       "expression and in the"
       "binding expression"))

(slide
 #:title fac-title
 #:layout fac-layout
 (code (letrec ([fac
                 (lambda (n)
                   (if (zero? n)
                       1
                       (* n (fac (- n 1)))))])
         (fac 10)))
 'next
 (blank)
 (blank)
 (para (code letrec) "hash the shape of" (code let)
       "but the binding structure of" (code local)))

(slide
 #:title fac-title
 #:layout fac-layout
 (code (let ([fac
              (lambda (n)
                (if (zero? n)
                    1
                    (* n (fac (- n 1)))))])
         (fac 10)))
 'next
 (blank)
 (para "Doesn't work, because" (code let) "binds" (code fac)
       "only in the body")
 'next
 (blank)
 (blank)
 (para "Still, at the point that we call" (code fac)
       ", obviously we have a binding for" (code fac)
       "...")
 'next
 (colorize (para #:align 'right "... so pass it as an argument!") BlueColor))

(define-syntax hcode
  (syntax-rules ()
    [(_ v) (parameterize ([code-colorize-enabled #t])
             (code v))]))

(define-syntax hylite
  (syntax-rules ()
    [(_ v) (colorize
            (parameterize ([code-colorize-enabled #f])
              (code v))
            RedColor)]))

(slide
 #:title fac-title
 #:layout fac-layout
 'alts
 (list
  (list
   (code (let ([facX
                (lambda (#,(hylite facX) n)
                  (if (zero? n)
                      1
                      (* n (fac (- n 1)))))])
           (facX #,(hylite facX) 10))))
  (list
   (code (let ([facX
                (lambda (#,(hylite facX) n)
                  (if (zero? n)
                      1
                      (* n (facX #,(hylite facX) (- n 1)))))])
           (facX #,(hylite facX) 10)))))
 'next
 (blank)
 (colorize (para #:align 'right "Wrap this to get" (code fac) "back...") BlueColor))

(slide
 #:title fac-title
 #:layout fac-layout
 (scode #,(hylite
           (let ([fac
                  (lambda (n)
                    #,(hcode
                       (let ([facX
                              (lambda (facX n)
                                (if (zero? n)
                                    1
                                    (* n (facX facX (- n 1)))))])
                         (facX facX #,(hylite n)))))])
             #,(hcode (fac 10)))))
 'next
 (blank)
 (para "Try this in the" (bt "HtDP Intermediate with Lambda")
       "language, click" (bt "Step"))
 'next
 (blank)
 (para "But the language we implement has only single-argument functions..."))

;; ------------------------------------------------------------
(encoding-part 6)

(slide
 #:title fac-title
 #:layout fac-layout
 (scode (let ([fac
               (lambda (n)
                 (let ([facX
                        (lambda (facX)
                          (lambda (n)
                            (if (zero? n)
                                1
                                (* n (#,(hylite (facX facX)) (- n 1))))))])
                   (#,(hylite (facX facX)) n)))])
          (fac 10)))
 'next
 (blank)
 (colorize
  (vr-append
   (current-line-sep)
   (para #:width client-w #:align 'right "Simplify:" (code (lambda (n) (let ([f ...]) ((f f) n)))))
   (para #:width client-w #:align 'right sym:implies (code (let ([f ...]) (f f))) "..."))
  BlueColor))

(slide
 #:title fac-title
 #:layout fac-layout
 (scode (let ([fac
               (let ([facX
                      (lambda (facX)
                        (lambda (n)
                          (if (zero? n)
                              1
                              (* n ((facX facX) (- n 1))))))])
                 (facX facX))])
          (fac 10))))

(slide
 #:title fac-title
 #:layout fac-layout
 (scode (let ([fac
               (let ([facX
                      (lambda (facX)
                        (code:comment "Almost looks like original fac:")
                        (lambda (n)
                          (if (zero? n)
                              1
                              (* n ((facX facX) (- n 1))))))])
                 (facX facX))])
          (fac 10)))
 'next
 (blank)
 (colorize
  (para #:align 'right
        "More like original: introduce a local binding for"
        (code (facX facX)) "...")
  BlueColor))


(slide
 #:title fac-title
 #:layout fac-layout
 (scode (let ([fac
               (let ([facX
                      (lambda (facX)
                        #,(hylite
                           (let ([fac (facX facX)])
                             #,(hcode
                                (code:comment "Exactly like original fac:"))
                             #,(hcode
                                (lambda (n)
                                  (if (zero? n)
                                      1
                                      (* n (#,(hylite fac) (- n 1)))))))))])
                 (facX facX))])
          (fac 10)))
 'next
 (blank)
 (vl-append
  (current-line-sep)
  (para (bt "Oops!") sym:emdash "this is an infinite loop")
  (para "We used to evaluate" (code (facX facX)) "only when"
        (code n) "is non-zero"))
 'next
 (blank)
 (colorize
  (para #:align 'right "Delay" (code (facX facX)) "...")
  BlueColor))

(slide
 #:title fac-title
 #:layout fac-layout
 (scode (let ([fac
               (let ([facX
                      (lambda (facX)
                        (let ([fac #,(hylite
                                      (lambda (x) 
                                        (#,(hcode (facX facX)) x)))])
                          (code:comment "Exactly like original fac:")
                          (lambda (n)
                            (if (zero? n)
                                1
                                (* n (fac (- n 1)))))))])
                 (facX facX))])
          (fac 10)))
 'next
 (blank)
 (para "Now, what about" (code fib) "," (code sum) ", etc.?")
 (colorize
  (para #:align 'right
        "Abstract over the" (hbl-append (code fac) (t "-specific"))
        "part...")
  BlueColor))

(slide
 #:title fac+rec-title
 (scode (define (mk-rec body-proc)
          (let ([fX
                 (lambda (fX)
                   (let ([f (lambda (x) 
                              ((fX fX) x))])
                     (body-proc f)))])
            (fX fX)))
        code:blank
        (let ([fac #,(hylite 
                      (mk-rec
                       (lambda (fac)
                         #,(hcode
                            (code:comment "Exactly like original fac:"))
                         #,(hcode
                            (lambda (n)
                              (if (zero? n)
                                  1
                                  (* n (fac (- n 1)))))))))])
          (fac 10))))

(slide
 #:title "Fibonnaci"
 (scode (let ([fib
               (mk-rec
                (lambda (fib)
                  (code:comment "Usual fib:")
                  (lambda (n)
                    (if (or (= n 0) (= n 1))
                        1
                        (+ (fib (- n 1))
                           (fib (- n 2)))))))])
          (fib 5))))

(slide
 #:title "Sum"
 (scode (let ([sum
               (mk-rec
                (lambda (sum)
                  (code:comment "Usual sum:")
                  (lambda (l)
                    (if (empty? l)
                        0
                        (+ (fst l)
                           (sum (rest l)))))))])
          (sum '(1 2 3 4)))))

(define (subscript p n)
  (hbl-append p (text (format "~a" n) `(subscript . ,(current-main-font)) (current-font-size))))

(define fae-1 (code _rhs))
(define fae-2 (code _body))
(define name-1 (code _name))
(define mk-rec (text  "mk-rec" `(italic . modern) (current-font-size)))

(define (ralign .  x)
  (apply item #:bullet (ghost bullet) x))

(slide
 #:title "Implementing Recursion"
 'alts
 (list
  (list
   (scode {letrec {[fac {lambda {n}
                          {if0 n
                               1
                               {* n
                                  {fac {- n 1}}}}}]}
            {fac 10}})
   (para #:width client-w "could be parsed the same as")
   (sscode {let {[fac 
                  {#,mk-rec
                   {lambda {fac}
                     {lambda {n}
                       {if0 n
                            1
                            {* n
                               {fac {- n 1}}}}}}}]}
             {fac 10}}))
  (list
   (ralign
    (code {letrec {[#,name-1 #,fae-1]}
            #,fae-2}))
   (para #:width client-w "could be parsed the same as")
   (ralign
    (code {let {[#,name-1 {#,mk-rec {lambda {#,name-1} #,fae-1}}]}
            #,fae-2}))
   (para #:width client-w "which is really")
   (ralign
    (code {{lambda {#,name-1} #,fae-2}
           {#,mk-rec {lambda {#,name-1} #,fae-1}}}))
   (vc-append
    (current-line-sep)
    (para #:width client-w "which, writing out" mk-rec ", is really")
    (ralign
     (scale
      (code {{lambda {#,name-1} #,fae-2}
             {{lambda {body-proc}
                {let {[fX {fun {fX}
                               {let {[f {lambda {x}
                                          {{fX fX} x}}]}
                                 {body-proc f}}}]}
                  {fX fX}}}
              {lambda {#,name-1} #,fae-1}}})
      0.45))))))
