#lang slideshow

(require slideshow/code
         "utils/colors.ss"
         "utils/utils.ss"
         "utils/alg.ss")

;; ----------------------------------------

(slide
 (titlet "Part 0"))

(define (align . l)
  (apply item #:bullet (ghost bullet) l))

(define fae-1 (code _rhs))
(define fae-2 (code _body))
(define id-1 (code _name))

(define mk-rec (text  "mk-rec" `(italic . modern) (current-font-size)))

(define (mk-rec-slide)
  (slide
   #:title "Defining Recursion by Expansion"
   (align
    (code {letrec {[#,id-1 #,fae-1]}
            #,fae-2}))
   (blank)
   (para #:width client-w "could be parsed the same as")
   (blank)
   (align
    (code {let {[#,id-1 {#,mk-rec {lambda {#,id-1} #,fae-1}}]}
            #,fae-2}))
   (blank)
   (para #:width client-w "which is really")
   (blank)
   (align
    (code {{lambda {#,id-1} #,fae-2}
           {#,mk-rec {lambda {#,id-1} #,fae-1}}}))))

(mk-rec-slide)

;; ----------------------------------------

(slide
 (titlet "Part 1")
 (titlet "Metacircular Recursion"))

;; ----------------------------------------

(define (loop-slide p r #:prefixes [prefixes null])
  (slide
   #:title "Recursive Binding"
   'alts
   (list prefixes)
   p
   'next
   (blank)
   r))

(define (para* . content)
  (apply para #:align 'center content))

(loop-slide
 (code (local [(define x 10)]
         (+ x 1)))
 (para* (tt "11")))
         
(loop-slide
 (code (local [(define (f x)
                 (f x))]
         (f 1)))
 (para* "infinite loop"))

(loop-slide
 (code (local [(define x x)]
         x))
 (para* (tt "#<undefined>")))
         
(loop-slide
 (code (local [(define x (list x))]
         x))
 (para* (tt "(list #<undefined>)")))
         
(loop-slide
 (code (local [(define f 
                 (lambda (x) (f x)))]
         (f 1)))
 (para* "infinite loop"))

(loop-slide
 (code (letrec ([f (lambda (x) (f x))])
         (f 1)))
 (para* "infinite loop"))

(loop-slide
 (code (letrec ([f 
                 (list
                  (lambda (x) ((first f) x)))])
         ((first f) 1)))
 (para* "infinite loop"))

(loop-slide
 #:prefixes (list (code {letrec {[x 10]}
                          {+ x 1}})
                  'next
                  (blank))
 (code (letrec ([val
                 (interp (numC 10) 
                         (extend-env (bind 'x val)
                                     env))])
         (interp (plusC (idC 'x) (numC 1))
                 (extend-env (bind 'x val)
                             env))))
 (para* "seems to work..."))

(loop-slide
 #:prefixes (list (code {letrec {[f {lambda {x}
                                      {f x}}]}
                          {f 1}})
                  (blank))
 (code (letrec ([val
                 (interp (lamC 'x (appC (idC 'f)
                                        (idC 'x)))
                         (extend-env (bind 'f val)
                                     env))])
         (interp (appC (idC 'f) (numC 1))
                 (extend-env (bind 'f val)
                             env))))
 (para* "contract failure"))

(loop-slide
 #:prefixes (list (code {letrec {[f {lambda {x}
                                      {f x}}]}
                          {f 1}})
                  (blank))
 (code (letrec ([new-env (extend-env (bind 'f (lambda ()
                                                val))
                                     env)]
                [val (interp (lamC 'x (appC (idC 'f)
                                            (idC 'x)))
                             new-env)])
         (interp (appC (idC 'f) (numC 1)) new-env)))
 (para* "works!"))

;; ----------------------------------------

(define meta-recur-title (hbl-append (titlet "Metacircular ") (code letrec)))

(slide
 #:title meta-recur-title
 (code
  (define-type Binding
    [bind (name : symbol)
          (val : (#,(hilite (code ->)) Value))])
  code:blank
  (define (lookup [n : symbol] [env : Env]) : Value
    (cond
     [(empty? env) (error 'lookup "free variable")]
     [else (if (symbol=? n (bind-name (first env)))
               #,(hbl-append (hilite (tt "("))
                             (code (bind-val (first env)))
                             (hilite (tt ")")))
               (lookup n (rest env)))]))))

;; ----------------------------------------

(slide
 (titlet "Part 2"))

(slide
 #:title "Expr Grammar"
 (grammar-table
  (list (nonterm "Expr")  eqls (nonterm "Num") (blank)
        (blank)          -or- (code {+ #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {- #,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code #,(nonterm "Sym")) (blank)
        (blank)          -or- (code {lambda {#,(nonterm "Sym")} #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {#,(nonterm "Expr") #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {let {[#,(nonterm "Sym") #,(nonterm "Expr")]} #,(nonterm "Expr")}) (blank)
        (blank)          -or- (code {if0 #,(nonterm "Expr") #,(nonterm "Expr") #,(nonterm "Expr")}) new-label
        (blank)          -or- (code {letrec {[#,(nonterm "Sym") #,(nonterm "Expr")]} #,(nonterm "Expr")}) new-label)))

;; ----------------------------------------

(slide
 #:title meta-recur-title
 (scale
  (code
   (define (interp [a : Expr] [env : Env]) : Value
     (type-case Expr a
       ....
       [letrecC (n rhs body)
                (letrec ([new-env
                          (extend-env
                           (bind n (lambda () val))
                           env)]
                         [val
                          (interp rhs new-env)])
                  (interp body new-env))])))
  0.9))

;; ----------------------------------------

(slide
 (titlet "Part 3")
 (titlet "Assignment-Based Recursion"))

(mk-rec-slide)

;; ----------------------------------------

(slide
 #:title "Defining Recursion by Expansion"
 (para #:width client-w "Another approach:")
 (align
  (scode (letrec ([fac
                   (lambda (n)
                     (if (zero? n)
                         1
                         (* n (fac (- n 1)))))])
           (fac 10))))
 (para sym:implies)
 (align
  (scode (let ([fac 42])
           (begin
             (set! fac
                   (lambda (n)
                     (if (zero? n)
                         1
                         (* n (fac (- n 1))))))
             (fac 10))))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(slide
 #:title "Implementing Recursion"
 (para "The" (code set!) "approach to definition works only"
       "when the defined language includes" (code set!))
 (blank)
 (para "But the" (code set!) "approach to implementation"
       "requires only that the implementation language includes"
       (code set!) "..."))

(define assign-recur-title (hbl-append (titlet "Assignment-Based ") (code letrec)))

(slide
 #:title assign-recur-title
 (code
  (define-type Binding
    [bind (name : symbol)
          (val : (#,(hilite (code boxof)) Value))])
  code:blank
  (define (lookup [n : symbol] [env : Env]) : Value
    (cond
     [(empty? env) (error 'lookup "free variable")]
     [else (if (symbol=? n (bind-name (first env)))
               (#,(hilite (code unbox)) (bind-val (first env)))
               (lookup n (rest env)))]))))

;; ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define (rcfae-short-code rec-code)
  (para
   #:width client-w
   (code
    (define (interp [a : Expr] [env : Env]) : Value
      (type-case Expr a
        ....
        [(code:line letrecC (n rhs body))
         #,rec-code])))))

(slide
 #:title assign-recur-title
 'alts~
 (list
  (list (rcfae-short-code (code ...)))
  (list (rcfae-short-code (code
                           ... (interp rhs env)
                           ... (interp body env) ...)))
  (list (rcfae-short-code (code
                           ... (interp rhs new-env)
                           ... (interp body new-env) ...)))
  (list (rcfae-short-code (code
                           (let ([new-env (extend-env
                                           (bind n ...)
                                           env)])
                             ... (interp rhs new-env)
                             ... (interp body new-env) ...))))
  (list (rcfae-short-code (code
                           (let ([new-env (extend-env
                                           (bind n (box ...))
                                           env)])
                             ... (interp rhs new-env)
                             ... (interp body new-env) ...))))
  (list (rcfae-short-code (code
                           (let ([new-env (extend-env
                                           (bind n (box (numV 42)))
                                           env)])
                             ... (interp rhs new-env)
                             ... (interp body new-env) ...))))
  (list (rcfae-short-code (code
                           (let ([b (box (numV 42))])
                             (let ([new-env (extend-env
                                             (bind n b)
                                             env)])
                               ... (interp rhs new-env)
                               ... (interp body new-env) ...)))))
  (list (rcfae-short-code (code
                           (let ([b (box (numV 42))])
                             (let ([new-env (extend-env
                                             (bind n b)
                                             env)])
                               ... (set-box! b (interp rhs new-env))
                               ... (interp body new-env) ...)))))
  (list (rcfae-short-code (code
                           (let ([b (box (numV 42))])
                             (let ([new-env (extend-env
                                             (bind n b)
                                             env)])
                               (begin
                                 (set-box! b (interp rhs new-env))
                                 (interp body new-env)))))))))

;; ----------------------------------------

(slide
 (titlet "Part 4")
 (titlet "Cyclic Data"))

(define (add-cycle-arrow p self)
  (refocus
   (pin-arrow-line (/ gap-size 2)
                   p
                   self cb-find
                   p (lambda (p q)
                       (define-values (x y) (lt-find p q))
                       (values x (+ y (pict-height self))))
                   #:start-angle (* pi -1/2)
                   #:end-angle (* pi 1/4)
                   #:color "orange"
                   #:line-width 2)
   p))

(slide
 #:title "Cycles"
 'alts
 (list
  (list
   (code
    (letrec ([f #,(let ([f (code f)])
                    (add-cycle-arrow
                     (code (lambda (x)
                             (#,f x)))
                     f))])
      ...)))
  (list
   (let ([hole (frame (ghost (tt "f")))])
     (add-cycle-arrow
      (code (closV 'x
                   (appC (idC 'f) (idC 'x))
                   (extend-env
                    (bind 'f (box #,hole))
                    mt-env)))
      hole)))))
